﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Model
{
    public class GameModel
    {
        #region Events
        public event EventHandler<NodeCoordinates> NodeUpgraded;
        public event EventHandler<(NodeCoordinates, NodeCoordinates)> RoadPlaced;
        public event EventHandler<List<(NodeCoordinates, NodeCoordinates)>> RoadsEnabled;
        public event EventHandler GameRolled;
        #endregion

        #region Private methods
        private NodeCoordinates GetCoordinates(int nodeId)
        {
            int r = nodeId / (Board.Size * 2);
            int q = (nodeId / 2) % Board.Size;
            Direction dir = (Direction)((nodeId) % 2);
            return new NodeCoordinates(r, q, dir);
        }
        #endregion

        public void NewGame(List<String> playerNames)
        {
            Board = new CatanBoard(playerNames);
        }
        public String GetPlayerName(int playerId)
        {
            return Board.Players[playerId].Name;
        }

        public void BuyCard(int playerId)
        {
            Board.BuyCard(Board.Players[playerId]);
        }

        public void SendEnabledRoads(int playerId)
        {
            RoadsEnabled?.Invoke(this, Board.EnabledRoads(Board.Players[playerId]));
        }

        public void SendEnabledRoadsFromNode(int playerId, NodeCoordinates pos)
        {
            RoadsEnabled?.Invoke(this, Board.EnabledRoadsFromNode(Board.Players[playerId], pos));
        }



        public void PlaceRoad((int FirstNodeId, int SecondNodeId) pos)
        {
            //System.Windows.MessageBox.Show(GetCoordinates(pos.FirstNodeId) + ", " + GetCoordinates(pos.SecondNodeId));
            Board.PlaceRoad((GetCoordinates(pos.FirstNodeId), GetCoordinates(pos.SecondNodeId)), Board.Players[CurrentPlayerId]);

            RoadPlaced?.Invoke(this, (GetCoordinates(pos.FirstNodeId), GetCoordinates(pos.SecondNodeId)));
        }

        public void UpgradeNode(NodeCoordinates node, int playerId)
        {
            bool canPlace;
            if(Board.Nodes[node.R, node.Q, (int) node.Dir].Building.Type == HouseType.NoBuilding)
                canPlace = Board.PlaceSettlement(node, Board.Players[playerId]);
            else
                canPlace = Board.UpgradeToCity(node, Board.Players[playerId]);

            if (canPlace && Board.InitialTurn)
                SendEnabledRoadsFromNode(playerId, node);

            if (canPlace)
                NodeUpgraded?.Invoke(this, node);
        }

        public void StepGame()
        {
            Board.StepGame();
        }

        public (int, int) Roll()
        {
            return Board.Roll();
        }

        private void SendResources(byte id)
        {
            Board.SendResources(id);
        }

        public CatanBoard Board { get; set; }
        public int CurrentPlayerId { get => Board.CurrentPlayerId; }
    }
}
