﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Model
{
    public class Player
    {
        private Dictionary<ResourceType, int> _resources;
        private int _roadLength;

        public Player()
        {
            AvailableSettlementCnt = 5;
            AvailableCityCnt = 4;
            AvailableRoadCnt = 15;
            _roadLength = 0;
            _resources = new()
            {
                // large values added for test purposes only
                { ResourceType.Brick, 100},
                { ResourceType.Lumber, 100 },
                { ResourceType.Wool, 100 },
                { ResourceType.Grain, 100 },
                { ResourceType.Ore, 100 },
            };
        }

        public void AddedSettlement()
        {
            Resources[ResourceType.Brick]--;
            Resources[ResourceType.Lumber]--;
            Resources[ResourceType.Wool]--;
            Resources[ResourceType.Grain]--;
            Score++;
            AvailableSettlementCnt--;
        }

        public void UpgradedToCity()
        {
            Resources[ResourceType.Grain] -= 3;
            Resources[ResourceType.Ore] -= 3;
            Score++;
            AvailableCityCnt--;
            AvailableSettlementCnt++;
        }

        public void RoadBuilt()
        {
            Resources[ResourceType.Brick]--;
            Resources[ResourceType.Lumber]--;
            AvailableRoadCnt--;
        }

        public void IncreaseArmy()
        {
            ArmySize++;
        }

        public void DevelopmentCardBought()
        {
            Resources[ResourceType.Ore]--;
            Resources[ResourceType.Grain]--;
            Resources[ResourceType.Wool]--;
        }

        public byte Id { get; set; }
        public String Name { get; set; }
        public int Score { get; set; }
        public int RoadLength { get => _roadLength; set => _roadLength = value; }
        public int ArmySize { get; set; }
        public Dictionary<CardType, int> DevelopmentCards { get; set; }
        public Dictionary<ResourceType, int> Resources { get => _resources; set => _resources = value; }
        public bool CanBuildSettlement { get => AvailableSettlementCnt > 0 && _resources[ResourceType.Lumber] > 0 && _resources[ResourceType.Brick] > 0 && _resources[ResourceType.Grain] > 0 && _resources[ResourceType.Wool] > 0; }
        public bool CanBuildCity { get => AvailableCityCnt > 0 && _resources[ResourceType.Grain] > 2 && _resources[ResourceType.Ore] > 2; }
        public bool CanBuildRoad { get => AvailableRoadCnt > 0 && _resources[ResourceType.Lumber] > 0 && _resources[ResourceType.Brick] > 0; }
        public bool CanBuyDevelopmentCard { get => Resources[ResourceType.Ore] > 0 && Resources[ResourceType.Grain] > 0 && Resources[ResourceType.Wool] > 0; }
        public bool HasLongestroad { get; set; }
        public bool HasLargestArmy { get; set; }
        public int AvailableSettlementCnt { get; private set; }
        public int AvailableCityCnt { get; private set; }
        public int AvailableRoadCnt { get; private set; }
    }
}
